// This file is part of the Dynpeak toolbox
// Copyright (C) 2012 - INRIA - Qinghua ZHANG - Serge Steer
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
function  peakind  = lhpeaks(sig,t, p)
//LHPEAK: LH peak detection

//peakind: indexes of detected peaks
//
//sig   : sampled LH signal 
//t     : vector of sample dates or time step
//p     : optionnal parameter struct
//
//Glossary---------------------------------------------------------------
//   - The height of a peak is the difference of the signal value at the
//     peak location and the minimal signal value
//   - The  magnitude of a peak
//   - The  relative magnitude of a peak  
//Method-----------------------------------------------------------------
// lhpeaks uses a multiple pass method to estimates the peak locations. 
  
// The first pass tries to sequentially find peaks that have  great
//     heights within a user specified time range consistent with the
//     maximal frequency (see firstPeaks).
  
// The second pass removes peaks whose height is small in comparison with
//     the median of the detected pulse heights: For each pulse, if, for a
//     given pulse, the ratio between its own height and the median height
//     is less than a threshold parameter (globalreltol), the pulse is
//     removed (see delGlobalRelSmallPeaks)
  
// The third pass removes the peaks which are not sufficiently high with
//     respect to the magnitude of the peaks immediately on the left and on the
//     right. More precisely if M is the peak magnitude, Ml and Mr the magnitude
//     of the left and right peaks, ml the minimum value of the signal in
//     the left interpeak interval and mr the minimum value of the signal in
//     the rigth inter peak interval, if
//     (M-Ml)*(M-Mr)<(Ml-min(ml,mr))*(Mr-min(ml,mr))*localreltol the peak is
//     rejected. See delLocalRelSmallPeaks.
  
// The fourth pass looks for local maxima located between two consecutive
//     peaks. These local maxima are considered to be peaks if they fullfill
//     third pass criterion. See recoverMissedPeaks
  
// The fifth pass removes peaks whose height is too small with respect to the
//     minimum signal value in the left and right inter peak intervals.
//     More precisely if h is the peak height, ml the minimum value of the
//     signal in the left interpeak interval and mr the minimum value of the
//     signal in the rigth inter peak interval, if
//     (h-ml)*(h-mr)<localabstol^2 the peak is rejected. See
//     delLocalAbsSmallPeaks
  
// The sixth pass removes peaks which are formed by only 3 points : the
//     peaks for which the immediately preceding and following samples are
//     local minima of the time series. See delThreePointPeaks.
// 
  
// Finally the algorithm may optionnaly take advantage of the uncertainty on
//     the signal values to make the inter peak interval length evolve more
//     regularily. The previous steps have set for each found peak a
//     property which indicates if a given peak is detected for all the
//     uncertainty range or not. The alternatePeakPositions which searchs
//     the peaks that are almost flat with respect to uncertainty.
//     lhpeaks_refine computes the "optimal" peaks locations over all
//     possible choices of peak presence or absence and possible peak
//     displacements.

//Revision history:------------------------------------------------------
//   Jan 2011: Initial version (Qinghua ZHANG, INRIA)
//   Feb 2011: (Qinghua ZHANG, INRIA)
//      - three point peaks detection included
//   July 2012:   (Serge Steer, INRIA)
//      - argument checking added
//      - a struct used to handle parameters
//      - modified to managed non regularily discretized signals.
//      - IPI regularity optimisation based on data uncertainty
// Reference:------------------------------------------------------------
//   "DynPeak : An algorithm for pulse detection and frequency analysis in
//      hormonal time series", Alexandre VIDALI, Qinghua ZHANG, Claire
//      MEDIGUE, Stéphane FABRE, Frédérique CLEMENT, Dec 2011
  fname="lhpeaks";
  if debug()<-1 then  debug_mode=max(0,-debug()-1);else debug_mode=0;end

  if type(sig)<>1|and(size(sig)<>1)|~isreal(sig) then
    error(msprintf(_("%s: Wrong type for argument %d: Real vector expected.\n"),fname,1))
  end
  sig=sig(:);
  N=size(sig,1)
  if type(t)<>1|~isreal(t)|and(size(t)<>1) then
    error(msprintf(_("%s: Wrong type for argument %d: Real array expected.\n"),fname,2))
  end
  if size(t,'*')==1 then
    t=(0:N-1)'*t;
  else
    if size(t,'*')<>N then
      error(msprintf(_("%s: Incompatible input arguments #%d and #%d: Same sizes expected.\n"),fname,1,2))
    end
    t=t(:)
  end
  if argn(2)>2 then
    params=lhpeaks_check_options(p,N,fname)
  else
    params=lhpeak_default_params()
  end
  period= params.period;
  imposedpeaks=params.imposedpeaks;
  //remove ignored points
  if params.ignoredpoints<>[] then
    sig(params.ignoredpoints)=%nan;
  end
  if params.selection<>[] then
    sig=sig(params.selection(1):params.selection(2));
    t=t(params.selection(1):params.selection(2));
    if imposedpeaks<>[] then
      imposedpeaks(imposedpeaks<params.selection(1)|imposedpeaks>params.selection(2))=[];
      imposedpeaks=imposedpeaks-params.selection(1)+1
    end
  end

  if params.ignoredpoints<>[] then
    k=find(isnan(sig))
    sig(k)=[];
    t(k)=[];
    for j=1:size(imposedpeaks,'*')
      imposedpeaks(j)=imposedpeaks(j)-size(find(k<imposedpeaks(j)),'*')
    end
  end

  localreltol=params.globalreltol^2
  sig=max(sig,params.detectionthreshold);
  if debug_mode>0 then debug_handles=dynpeak_init_debug();end
 
  [smax, indmax] = max(sig);
  peakind=indmax; //assume a peak at the max signal value 
  
  if indmax<N then
    //search peaks between the max signal value and the end of the signal
    ind=firstPeaks(sig(indmax:$),t(indmax:$), period,params.uncertainty)
    if ind<>[] then
      peakind = [peakind;
                 indmax-1+ind];
    end
  end
  
  if indmax>1 then
    //search peaks between the beginning of the signal ant its the max value
    ind = firstPeaks(sig(indmax:-1:1),cumsum([0;-diff(t(indmax:-1:1))]),period, ...
                     params.uncertainty);
    if ind<>[] then
      peakind = [indmax+1-ind($:-1:1);
                 peakind]
    end
  end
  Q=ones(peakind);

  if debug_mode>0 then debug_mode=dynpeak_show_debug("First detection",peakind,Q);end

  test=%t;
  if test
    // Recover missed peaks thld2
    [peakind,Q] = recoverMissedPeaks(sig, peakind,localreltol,params.uncertainty);
    if debug_mode>0 then debug_mode=dynpeak_show_debug("Recovered",peakind,Q);end
  end

  // Remove too small peaks - global median; thld1
  [peakind,Q] = delGlobalRelSmallPeaks(sig, peakind,params.globalreltol,params.uncertainty);
  if  debug_mode>0 then debug_mode=dynpeak_show_debug("Global Median",peakind,Q),end

  // Remove too small peaks - local relative magnitude; thld2
  [peakind,Q] = delLocalRelSmallPeaks(sig, peakind,localreltol,params.uncertainty);
  if  debug_mode>0 then debug_mode=dynpeak_show_debug("Local Rel Mag",peakind,Q),end
 
  // (Added Dec 2010) Remove too small peaks - local absolute magnitude  AbsMagThreshold
  [peakind,Q] = delLocalAbsSmallPeaks(sig, peakind,params.localabstol,params.uncertainty);
  if debug_mode>0 then debug_mode=dynpeak_show_debug("Local Abs Mag",peakind,Q),end

  
  [peakind,Q] = delThreePointPeaks(sig,t, peakind,params.threepointtol,params.uncertainty);
  if debug_mode>0 then  debug_mode=dynpeak_show_debug("Three points",peakind,Q),end

//  [peakind,Q] = delOutliers(sig,t, peakind,params.threepointtol,params.uncertainty);
//  if debug_mode>0 then  debug_mode=dynpeak_show_debug("Outliers",peakind,Q),end
 
  [km,alt]=alternatePeakPositions(sig,peakind,params.uncertainty)
  if debug_mode>0 then debug_mode=dynpeak_show_debug("Alternate position",peakind,Q),end

  for k=size(imposedpeaks,'*'):-1:1
    j=find(peakind==imposedpeaks(k),1)
    if j==[] then
      l=find(imposedpeaks(k)<peakind,1)
      peakind=[peakind(1:l-1);imposedpeaks(k);peakind(l:$)]
      Q=[Q(1:l);1;Q(l+1:$)]
      l=find(peakind(km)>imposedpeaks(k))
      if l<>[] then km(l)=km(l)+1,end
    else
      Q(j)=1
      i=find(km==j)
      if i<>[] then
        alt(i)=null()
        km(i)=[]
      end
    end
  end

  p=peakind
  [peakind,Q]=lhpeaks_refine(peakind,t,km,alt,Q)
  if debug_mode>0 then debug_mode=dynpeak_show_debug("Refine",peakind,Q),end
  
  //update index to take care of ignored points
  ignoredpoints=params.ignoredpoints;
  if params.selection<>[] then
    peakind=peakind+params.selection(1)-1
  end

  if ignoredpoints<>[] then
    ignoredpoints=gsort(ignoredpoints,'g','i');
    for k=size(ignoredpoints,'*'):-1:1
      l=find(peakind>=ignoredpoints(k));
      if l<>[] then peakind(l)=peakind(l)+1;end
    end
  end
  if debug_mode>0 then dynpeak_end_debug();end
endfunction

function [peakind,Q] = delGlobalRelSmallPeaks(sig, peakind,rthld1,uncertainty)
// Remove too small peaks - global median;
  Q=Q
  Qs=Q;
  smin = min(sig);
  Hpeak = sig(peakind);
  pmed = median(Hpeak-smin);
   //peaks to be kept if upper position is retained
  upper=LH_upper(Hpeak)-smin>rthld1*pmed
  //peaks to be kept if lower position is retained
  lower=LH_lower(Hpeak)-smin>rthld1*pmed
  // upper option peaks are considered probable
  Q(upper)=0

  //lower option peaks are considered certain if they were not
  //previously tagged as uncertain
  Q(lower)=ones(Q(lower)).*Qs(lower)
  
  //upper option peaks  includes upper option peaks 
  //so all upper option peaks are retained 
  peakind = peakind(upper);
  Q=Q(upper);
endfunction

function [peakind,Q] = delLocalRelSmallPeaks(sig, peakind,rthld2,uncertainty)
// Remove too small peaks - local relative magnitude;
  Q=Q
  Np = length(peakind);
  pmask = %t(ones(Np,1));
  for k=2:(Np-1)
    smin_left = min(sig((peakind(k-1)+1):(peakind(k)-1)));//min on the left
    smin_right = min(sig((peakind(k)+1):(peakind(k+1)-1)));//min on the right
     
    //handle case of two consecutive peaks
    if smin_left==[] then 
      smin=smin_right;                                                
    elseif smin_right==[] then
      smin=smin_left;
    else
      smin = min(smin_left,smin_right);
    end
    cv=CV_LH(sig(peakind(k-1:k+1)),uncertainty)
    //hypothesis that maximize peak height
    sk=sig(peakind(k))+cv(2)
    sk_left=sig(peakind(k-1))-cv(1)
    sk_right=sig(peakind(k+1))-cv(3)
    to_remove_upper=(sk-smin_left)*(sk-smin_right)< rthld2*(sk_left-smin)*(sk_right-smin)
    //remove peaks that are too small even with maximizing hypothesis
    pmask(k)=~to_remove_upper

    //hypothesis that minimize peak height
    sk=sig(peakind(k))-cv(2);
    sk_left=sig(peakind(k-1))+cv(1);
    sk_right=sig(peakind(k+1))+cv(3);
    to_remove_lower=(sk-smin_left)*(sk-smin_right)< rthld2*(sk_left-smin)*(sk_right-smin);
    if to_remove_lower then  Q(k)=0,end 
  end
  peakind = peakind(pmask);
  Q=Q(pmask)
endfunction
//--------------------------------
function  [peakind,Q] = recoverMissedPeaks(sig, peakind,rthld2,uncertainty)
   Q=Q
// Recover missed peaks
  for k=1:10
    [pind,Q1] = missedPeaks(sig, peakind,rthld2,uncertainty);
    if pind==[] then break,end
    [peakind,ind] = gsort([peakind; pind],'g','i');
    Q=[Q;Q1];Q=Q(ind);
  end
endfunction
//--------------------------------
function [pind2,Q] = missedPeaks(sig, peakind,rthld2,uncertainty)
//search for a missed peak between two peaks detected by the firstPeak pass
//an intermediate  peak candidate is retained if it is relatively "high enough"
//compared to the two surrounding peaks.
  Np = length(peakind);
  pind2=[]
  Q=[];
  //loop on interpeak intervals
  for k=1:(Np-1)
    //search for missed peak between peaks k and k+1
    sig_seg=sig(peakind(k):peakind(k+1));
    if size(sig_seg,'*')<5 then continue,end
    cv=CV_LH(sig_seg,uncertainty);
    d=diff(sig_seg);
    //-------find local max (d(i)>0&d(i+1)<0) in sig_seg -----------------
    //maximize local max detection using uncertainty
    upper=d(1:$-1)+cv(1:$-2)>0&d(2:$)-cv(2:$-1)<0;
    upper([1 $])=%f; //avoid double detection of "flat peaks" on boundary peaks
    //minimize local max detection  using uncertainty
    lower=d(1:$-1)-cv(1:$-2)>0&d(2:$)+cv(2:$-1)<0;
    
    p=upper;//&lower;
    if ~or(p) then continue,end
    kp=find(upper|lower)+1;
    //-------compute high criterion for each local max-------------------
    crtu=zeros(kp);
    crtl=crtu;
    for i=1:size(kp,'*')
      //kminlr index of the 2 local minima on the left and the right of the
      //current local max
      si=sig_seg(kp(i)); //intermediate peak candidate
      cvi=cv(kp(i));  //uncertainty for intermediate peak candidate
      //maximized peak criterion
      dp=d-cv(1:$-1);
      kminlr=kp(i)+[1-find(dp(kp(i)-1:-1:1)<0,1),-1+find(dp(kp(i):$)>0,1)];
      slr=sig_seg(kminlr);// signal local min values
      cvlr=cv(kminlr) ;//uncertainty for  signal local min values
      crtu(i)=prod((si+cvi)-(slr-cvlr));//maximized criterion
     //minimized peak criterion
      dp=d+cv(1:$-1);
      kminlr=kp(i)+[1-find(dp(kp(i)-1:-1:1)<0,1),-1+find(dp(kp(i):$)>0,1)];
      slr=sig_seg(kminlr);// signal local min values
      cvlr=cv(kminlr) ;//uncertainty for  signal local min values
      crtl(i)=prod((si-cvi)-(slr+cvlr));//minimized criterion
    end
    //absolute minimum within the interval
    [smin,kmin]=min(sig_seg);

    //-------select local max that has the maximum  criterion 
    [spmax,i]=max(crtl) //try minimized criterion
    //compare with peaks k and k+1 criterion
    if spmax > rthld2*(sig_seg(1)-smin)*(sig_seg($)-smin) then
      pind2=[pind2;peakind(k)+kp(i)-1]
      Q=[Q;1] // mark this peak as certain
    else
      [spmax,i]=max(crtu) //try maximized criterion
      if spmax > rthld2*(sig_seg(1)+cv(1)-smin+cv(kmin))*(sig_seg($)+cv($)-smin+cv(kmin)) then
        pind2=[pind2;peakind(k)+kp(i)-1]
        Q=[Q;0] // mark this peak as removable
      end
    end
  end

endfunction
//--------------------------------
function [peakind,Q] = delLocalAbsSmallPeaks(sig, peakind,athld,uncertainty)
  Q=Q
// (Added Dec 2010) Remove too small peaks - local absolute magnitude
  Np=size(peakind,'*');
  if Np<2 then return,end

  pmasku = %t(ones(peakind));
  pmaskl = %t(ones(peakind));
  
  // First peak
  //hypothesis that maximize peak height
  smin_right = min(LH_lower(sig((peakind(1)+1):(peakind(2)-1))));
  s=LH_upper(sig(peakind(1)));
  pmasku(1) = s-smin_right > athld;
  //hypothesis that minimize peak height
  smin_right = min(LH_upper(sig((peakind(1)+1):(peakind(2)-1))));
  s=LH_lower(sig(peakind(1)));
  pmaskl(1) = s-smin_right > athld;

  // Last peak
  //hypothesis that maximize peak height
  smin_left = min(LH_lower(sig((peakind(Np-1)+1):(peakind(Np)-1))));
  s=LH_upper(sig(peakind(Np)));
  pmasku(Np) = s-smin_left > athld;
  //hypothesis that minimize peak height
  smin_left = min(LH_lower(sig((peakind(Np-1)+1):(peakind(Np)-1))));
  s=LH_upper(sig(peakind(Np)));
  pmasku(Np) = s-smin_left > athld;

  //other points
  //hypothesis that maximize peak height
  for k=2:(Np-1)
    left_points=(peakind(k-1)+1):(peakind(k)-1)
    //hypothesis that maximize peak height
    smin_left = min(LH_lower(sig(left_points)));
    right_points=(peakind(k)+1):(peakind(k+1)-1)
    smin_right = min(LH_lower(sig(right_points)));
    sk=LH_upper(sig(peakind(k)))
    if (sk-smin_left)*(sk-smin_right) < athld^2;
      pmasku(k) = %f;
    end 
  end
  //hypothesis that minimize peak height
  for k=2:(Np-1)
    left_points=(peakind(k-1)+1):(peakind(k)-1)
    //hypothesis that maximize peak height
    smin_left = min(LH_upper(sig(left_points)));
    right_points=(peakind(k)+1):(peakind(k+1)-1)
    smin_right = min(LH_upper(sig(right_points)));
    sk=LH_lower(sig(peakind(k)))
    if (sk-smin_left)*(sk-smin_right) < athld^2;
      pmaskl(k) = %f;
    end 
  end
  Q(~pmasku|~pmaskl)=0
  pmask=pmasku|pmaskl
  peakind = peakind(pmask);
  Q=Q(pmask)
endfunction

function [peakind,Q] = delThreePointPeaks(sig,t, peakind,p3ptThreshold,uncertainty)
  Q=Q
  Npind = length(peakind);
  pmasku=%t(ones(peakind));
  pmaskl=%t(ones(peakind));
  //hypothesis that maximize 3pt peak  detection
  for kk=1:Npind
    if peakind(kk)<3 | peakind(kk)>length(sig)-2 then
      //skip boundary peaks
      continue
    end
    pk=peakind(kk);
    //hypothesis that maximize 3pt peak  detection
    s_max([1 3 5])=LH_upper(sig(pk+[-2 0 2]));
    s_max([2 4])=LH_lower(sig(pk+[-1 1]));
    if s_max([1 3])>s_max(2)&s_max([3 5])>s_max(4) then // \/\/
      //pk is the index of a 3pt peak
      d=diff(s_max)./diff(t(pk-2:pk+2));
      //sharpness coefficient
      ratio=abs(0.5*(d(1)-d(4))/sqrt(-d(2)*d(3)));
      pmasku(kk) = ~(ratio>=p3ptThreshold);
    end
    
    //hypothesis that minimize 3pt peak  detection
    s_min([1 3 5])=LH_lower(sig(pk+[-2 0 2]));
    s_min([2 4])=LH_upper(sig(pk+[-1 1]));
    if s_min([1 3])>s_min(2)&s_min([3 5])>s_min(4) then // \/\/
      //pk is the index of a 3pt peak
      d=diff(s_min)./diff(t(pk-2:pk+2))
      //sharpness coefficient
      ratio=abs(0.5*(d(1)-d(4))/sqrt(-d(2)*d(3)));
      pmaskl(kk) = ~(ratio>=p3ptThreshold);
    end
  end
  Q(~pmasku|~pmaskl)=0
  pmask=pmasku|pmaskl
  peakind = peakind(pmask);
  Q=Q(pmask)

endfunction

function [peakind,Q] = delThreePointPeaks_new(sig,t, peakind,p3ptThreshold,uncertainty)
  Q=Q
  Npind = length(peakind);
  pmask=%t(ones(peakind));
  for kk=1:Npind
    if peakind(kk)<3 | peakind(kk)>length(sig)-2 then
      //skip boundary peaks
      continue
    end
    pk=peakind(kk);
    dt=diff(t(pk-2:pk+2));

    //Attention il faudrait exclure des 3 points pics ceux tels que 
    //sig(pk-1) sig(pk+1) sont tres differents
    //     /\
    //    /
    //  \/
    //ou ce qui doit etre equivalent mean_slopes(2) tres different de mean_slopes(3)
    //hypothesis that maximize segment slopes
    sig_max([1 3 5],1)=LH_upper(sig(pk+[-2 0 2]));
    sig_max([2 4],1)=LH_lower(sig(pk+[-1 1]));
    slopes_max=diff(sig_max)./dt;
    if slopes_max(1)<0&slopes_max(4)>0 then // shape may be \/\/ 
      mean_slopes=abs(diff(sig(pk-2:pk+2)))./dt;
      //Too dissymetric shapes are not considered as  3 point peak candidates
      //check symetrie
      if abs(mean_slopes(2)-mean_slopes(3))/max(mean_slopes(2:3))<p3ptThreshold then
        //Check if segments [pk-1 pk] and [pk pk+1] have significantly higher
        //absolute value slopes than segments [pk-1 pk-2] and [pk+1 pk+2].
        //this may suggest pk beeing an erroneous value.
        sharpness=(0.5*(mean_slopes(4)+mean_slopes(1)))/max(mean_slopes(2:3))
        if sharpness>p3ptThreshold then 
          //may be a 3 point peak. Mark it as uncertain
          Q(kk)=0;                       
          //hypothesis that minimize segment slopes
          sig_min([1 3 5],1)=LH_lower(sig(pk+[-2 0 2]));
          sig_min([2 4],1)=LH_upper(sig(pk+[-1 1]));
          slopes_min=diff(sig_min)./dt;
          if slopes_min(1)<0&slopes_min(4)>0 then // shape is \/\/  in any case
            
            //peak is a 3 point in any cases. Ask for removal
            pmask(kk)=%f
        end
      end
      end
    end
  end

  peakind = peakind(pmask);
  Q=Q(pmask)

endfunction

function [peakind,Q] = delOutliers(sig,t, peakind,p3ptThreshold,uncertainty)
  Q=Q
  Npind = length(peakind);
  pmask=%t(ones(peakind));
  //checking for \/\
  //                \
  for kk=1:Npind
    if peakind(kk)<3 | peakind(kk)>length(sig)-2 then
      //skip boundary peaks
      continue
    end
    pk=peakind(kk);
    slope_left=[sig(pk-1)-LH_upper(sig(pk-2)), sig(pk-1)-LH_lower(sig(pk-2))]/(t(pk-1)-t(pk-2));
    slope_right=[LH_lower(sig(pk+1))-sig(pk-1),LH_upper(sig(pk+1))-sig(pk-1)]/(t(pk+1)-t(pk-1));
    if slope_left(1)<0&slope_right(1)<0 then //max slopes are >0 peak can be an outlier point
      Q(kk)=0;                       
      //check min slopes
      if slope_left(2)<0&slope_right(2)<0 then
        //peak is an outlier point in any cases
        pmask(kk)=%f
      end
    end
  end
  //checking for /\/\ (two peaks at k and k+2 with alocal min a k+1)
  //remove if the elimination of middle point gives a regular decrease /\
  //                                                                     \
   for kk=Npind-1:-1:1 //loop from right to left to handle case /\/\/\
    if peakind(kk)<3 | peakind(kk)>length(sig)-2 then
      //skip boundary peaks
      continue
    end
    pk=peakind(kk);
    if peakind(kk+1)==pk+2 then // /\/\
      if sig(pk+2)<sig(pk)&LH_upper(sig(pk+1))>sig(pk+2) then
        //force left point if it has been previously marked uncertain
        Q(kk)=1; 
        pmask(kk)=%t;
        //remove right peak
        pmask(kk+1)=%f
      end
    end
  end
 

  peakind = peakind(pmask);
  Q=Q(pmask)

endfunction

function peakind = firstPeaks(sig,t,period,uncertainty)
//sig         : part of a lh signal starting at a peak location
//t           : associated time instants in min
//period      : time horizon used for next peak search
//uncertainty : not yet used here  
//Method
  
//  The algorithm looks for the sequence of pulse which have a larger height
//  within a given time range:

//  Starting with indmax=1 (the first point of  sig  is assumed
//  to be a peak) the algorithm iterates up to the end of sig the
//  following sequence: 
  
//  1 - The algorithm first looks for the mimimun signal point in the
//      interval [t(indmax), t(indmax+period)],while the min point is found
//      at the right border of the interval the interval is increased by one
//      time step. The index of the minimum point is stored into
//      indmin. If indmin is the last time index the algorithm terminates
//  2 - If the maximum signal value in the interval ]t(indmax) t(indmin)[ is
//      greater than sig(indmax) then indmax is updated to this new maximum
//      index and the algorithm restart at step 1.
  
//  3 - The algorithm looks for the max point in the [t(indmin),
//      t(indmin+period)], while the max point is found at the right border
//      of the interval the interval is increased by one time step. indmax
//      is set to this max point index. If indmax is the last time index the
//      algorithm terminates. in the other case a new peak is added and the
//      algorithms restart a new sequence at step 1.
 
  
  N = length(sig);
  indmax=1;//sig(indmax) is a max point
  peakind = [];//vector of detected  peaks locations
  
  while %t //peak detection loop
    //looking for the min point in the [t(indmax) t(indmax)+period] interval
    inde=find(t>t(indmax)+period,1);
    if inde==[] then inde=N,else inde=inde-1;end
    [smin, indmin] = min(sig((indmax+1):inde));
    indmin = indmin+indmax;
    //check if sig continue to decrease after t(indmax)+period (serge june 14 2012)
    while indmin<N&sig(indmin+1)<=sig(indmin) then indmin=indmin+1;end
    if indmin==N then 
      //min reached on the boundary no maximum can follow, job finished
      break
    end
 
    // Check better peak candidate found in the interval ]t(indmax),t(indmin)]
    [exmax, exind] = max(sig((indmax+1):indmin));
    if exmax>sig(indmax) then
      //a better peak candidate found. Update previously detetected peak location
      indmax = exind + indmax;
      if ~isempty(peakind) then ; 
        peakind($) = indmax; // Update the last index
      end
      continue
    end
    
    // /\/ detected
    //looking for the max point in the [t(indmin) t(indmin)+period] interval
    //that can be the next peak candidate
    indmaxlast = indmax;
    inde=find(t>t(indmin)+period,1);
    if inde==[] then inde=N,else inde=inde-1;end
    [smax, indmax] = max(sig((indmin+1):inde));
    indmax = indmax+indmin;
    
    // check if sig continue to increase after t(indmin)+period (serge june 14 2012)
    while indmax<N&sig(indmax+1)>=sig(indmax) then indmax=indmax+1;end
   
    if indmax<N then 
      // /\/\ detected, indmax is the location of a peak
       peakind = [peakind; indmax];
    else
      // Left border, it is impossible to know if the t(N) is a peak
      // look for a smaller peak in [t(indmaxlast) t(N-1)].
      
      //search for a local min on the left of t(N)
      indmin1=N
      while indmin1>indmin&sig(indmin1-1)<sig(indmin1) then indmin1=indmin1-1;end
      if indmin1<=indmin then 
        //there is no possible peak ( /\ ) in [t(indmin) t(N)], job finished
        break
      end
      //search for the max point in [t(indmin) t(indmin1)]
      [smax, indmax] = max(sig((indmin+1):indmin1-1));
      indmax=indmax+indmin
      // /\/\ detected, indmax is the location of a peak
      peakind = [peakind; indmax];
    end
  end
endfunction

function [km,alt]=alternatePeakPositions(sig,peakind,uncertainty)
  cv=CV_LH(sig,uncertainty)
  if peakind(1)==1 then 
    kleft=find(LH_lower(sig(peakind(2:$))) < LH_upper(sig(peakind(2:$)-1)));
    if kleft<>[] then kleft+1;end
  else 
    kleft=find(LH_lower(sig(peakind)) < LH_upper(sig(peakind-1)));
  end
  //point following peak can be considered as an alternative peak
  //position (2 point flat peak)
  if peakind($)==size(sig,'*') then 
    kright=find(LH_lower(sig(peakind(1:$-1))) < LH_upper(sig(peakind(1:$-1)+1)));
  else
    kright=find(LH_lower(sig(peakind)) < LH_upper(sig(peakind+1)));
  end

  //km is the indices of "two points peaks" in sig
  km=union(kleft,kright)
  alt=list();
  
  //set possible peak displacements
  for j=1:size(km,'*')
    v=0;
    if or(km(j)==kleft) then v=[v -1];end
    if or(km(j)==kright) then v=[v 1];end
    alt(j)=v;
  end
endfunction
