// This file is part of the Dynpeak toolbox
// Copyright (C) 2012 - INRIA - Serge Steer
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
function [peakind,Q]=lhpeaks_refine(peakind,T,km,vars,Q)
//peakind     : vector of index of detected peaks in the signal
//km          : vector of index (relative to peakind) of peaks that can be moved
//vars        : list. vars(i) gives the admissible move for km(i)
//Q           : vector. If Q(j)==0 then peakind(j) can be removed 
//                                 else peak of index peakind(j) can only be moved

  max_nopt=20000; //max number of local combinaison allowed
  
  
  modifiable=union(km,find(Q<>1)); //index of peaks that can be moved or removed
  if modifiable==[] then return,end
  //find consecutive sequence of "modifiable peaks"
  dm=diff(modifiable);
  im=1;
  //seq_start and seq_end will store index relative to the begining and
  //end of the consecutive sequences of "modifiable peaks"
  seq_start=[];seq_end=[];
  while im<=size(modifiable,'*')
    seq_start=[seq_start im];
    //count how many consecutive modifiable peaks following this one
    k=find(dm(im:$)>1,1);
    if k==[] then
      seq_end=[seq_end,size(modifiable,'*')];
      break
    else
      seq_end=[seq_end,im-1+k];
    end
    im=im+k;
  end
  //optimize each sequence of modifiable peaks individually to avoid
  //combinatorial explosion as well as excessive smoothing.
  
  for i=size(seq_start,'*'):-1:1
    //option applicable to the peaks in this sequence
    peakoptions=list() ;  
    nopt=1
    for k=seq_start(i):seq_end(i)
      po=0
      if or(modifiable(k)==km) then
        po=vars(find(km==modifiable(k)))
      end
      if Q(modifiable(k))==0 po=[po %nan]; end
      peakoptions($+1)=po;
      nopt=nopt*size(po,'*')
    end
    //determine optimal position of uncertain peaks in the ith sequence

    
    indi=modifiable(seq_start(i):seq_end(i))';
    [opt_peak_index,keep]=optloc(peakind,T,modifiable,[seq_start(i),seq_end(i)],peakoptions);
    peakind(indi(keep))=opt_peak_index;

    if size(find(keep),'*')<>size(indi,'*') then
      removed=1:size(indi,'*');removed(keep)=[];
      Q(indi(removed))=[];
      peakind(indi(removed))=[];
      nr=size(removed,'*');

      [v,kr]=intersect(modifiable,indi(removed))
      for ir=1:size(kr,'*')
        modifiable(kr(ir):$)=modifiable(kr(ir):$)-1;
      end
      modifiable(kr)=[];
    end
  end
endfunction

function [opt_peak_index,opt_keep]=optloc(peakind,T,modifiable,seq,peakoptions)
//peakind     : full set of peaks index within signal
//T           : the vector of the signal observation dates
//modifiable  : index of uncertain peaks (peaks that can be moved or
//              removed) relative to peakind
//seq         : [seq_start seq_end]: index relative to modifiable of 
//               the sequence of consecutive uncertain peaks to be optimized
//peakoptions : peakoptions(i) if the vector of admissible modifications 
//              that can be applied on the indi(i) peak
//The algorithm finds the optimal peaks position in the given sequence of consecutive
//               uncertain peaks checking all possible combinations. The
//               criterion is the norm of the whole set of IPI's to the a
//               least square cubic interpolation of this set. The weight of
//               the IPI's associated to uncertain peaks that follow the
//               current sequence is reduced.
  
  indi=modifiable(seq(1):seq(2))';
  if indi==[] then return,end
  d=gencomb(peakoptions)
  np=size(peakind,'*')

  //search the hypothesis that keeps less peaks
  ndel=0;
  for j=1:size(d,1)
    ndel=max(ndel,size(find(isnan(d(j,:))),'*'));
  end
  
  //Common  interpolation order for all hypothesis
  nord=min(3,max(np-ndel-3,1)); //max interpolation order
  //loop on hypothesis
  opt_peak_index=peakind(indi);
  opt_keep=[];
  m=%inf;
  for j=1:size(d,1)
    hypo=d(j,:);
    keep=~isnan(hypo);
    kept_peak_index=indi(keep);
    kept_peak_displacement=hypo(keep)';
    new_peak_index=peakind(kept_peak_index)+kept_peak_displacement;
    sel=[peakind(1:indi(1)-1);new_peak_index;peakind(indi($)+1:$)];
    n1=indi(1)-1+size(new_peak_index,'*');
    ns=size(sel,'*');
    loc=T(sel);
      
    //compute least square weight of IPI's interpolation
    //set a weight array decreasing on the two sides of the sequence no
    //to regularize too much
    //and reduced  for uncertain IPI's on the left (not yet optimized)
    w=ones(loc);
    w(1:indi(1)-1)=linspace(0.3,1,indi(1)-1)';
    w(n1+1:$)=linspace(1,0.3,ns-n1)';
    //reduce weight for uncertain IPI's in the sequence and on the left (not yet optimized)
    for k=indi(1):n1,w(k)=0.2*w(k);end
    for k=1:indi(1)-1
     if or(sel(k)==peakind(modifiable)) then w(k)=0.2*w(k),end
    end

    if size(loc,'*')<4 then 
      //add fake peaks on the left and on the right
      loc=[loc(1)-(T(peakind(2))-T(peakind(1)));loc;loc($)+T(peakind($))-T(peakind($-1))];
      w=[0.2;w;0.2];
      if size(loc,'*')<4 then 
        //too few peaks with such an hypothesis the approximation curve
        //mathes exactly the IPI's. The criterion is 0. Exclude this
        //hypothesis of the optimization set.
        continue,
      end
    end
    v=objective(loc,nord,w);
    if v<m then
      m= v;
      opt_keep=keep;
      opt_peak_index=new_peak_index;
    end
  end
  if opt_keep==[] then 
    //all sequences have been excluded retain all peaks
    opt_keep=1:size(indi,"*")
    opt_peak_index=peakind(indi);
  end
endfunction

function r=gencomb(vars)
  v=vars(1);
  r=v(:);
  for k=2:size(vars)
    v=vars(k);v=v(:)
    n=size(v,1)
    r=[ones(n,1).*.r v.*.ones(size(r,1),1)]
  end
endfunction

function c=objective(loc,nord,w)
  y=diff(loc);
  c=norm(y-dynpeak_smooth_IPI(y,nord,w(1:size(y,'*'))))/size(y,'*');
endfunction
